// Copyright 2002 Macromedia, Inc. All rights reserved.

// *************** GLOBALS VARS *****************

// ***************** LOCAL FUNCTIONS  ******************

//--------------------------------------------------------------------
// FUNCTION:
//   queueDefaultDocEdits
//
// DESCRIPTION:
//   This function is called before the docEdits are applied to the
//   page to allow any default doc edits to be added.  We will add
//   the language tag.
//
// ARGUMENTS:
//   none
//
// RETURNS:
//   nothing
//--------------------------------------------------------------------

function queueDefaultDocEdits()
{
  var partList = dw.getParticipants("PageDirective_lang");

  if (!partList || (partList.length == 0))  // does not currently exist
  {
    var paramObj = new Object();
    paramObj.Language = "C#";
    dwscripts.queueDocEditsForParticipant("PageDirective_lang", paramObj);
  }
  else
  {
    dwscripts.queueParticipantInfo("PageDirective_lang", partList[0].participantNode);
  }
}

//--------------------------------------------------------------------
// FUNCTION:
//   encodeDynamicExpression
//
// DESCRIPTION:
//   This function prepares a dynamic expression for insertion onto
//   the page.  It is assumed that this expression will be used
//   within a larger dynamic statement, therefore all server markup
//   is stripped.
//
// ARGUMENTS:
//   expression - string - the dyanmic expression to encode
//
// RETURNS:
//   string
//--------------------------------------------------------------------

function encodeDynamicExpression(expression)
{
  var retVal = "";
  expression = expression.toString();

  if (hasServerMarkup(expression))
  {
    retVal = trimServerMarkup(expression);
  }
  else
  {
    var parameters = expression.match(/(true|false|[-]?\d+[\.]?\d*)/);
    if (parameters && parameters[0].length == expression.length) // matches, return exact
    {
      retVal = expression;
    }
    else
    {
      if (!dwscripts.isQuoted(expression))
      {
        retVal = "\"" + dwscripts.escQuotes(expression) + "\"";
      }
      else
      {
        retVal = expression;
      }
    }
  }

  return retVal;
}


//--------------------------------------------------------------------
// FUNCTION:
//   decodeDynamicExpression
//
// DESCRIPTION:
//   This function prepares a dynamic expression for display within
//   a dialog box.  Quotes are removed,a nd server markup is re-added.
//
// ARGUMENTS:
//   expression - string - the dynamic expression to prepare for display
//
// RETURNS:
//   string
//--------------------------------------------------------------------

function decodeDynamicExpression(expression)
{
  var retVal = "";

  expression = dwscripts.trim(expression.toString());
  var unquoted = dwscripts.trimQuotes(expression);

  var parameters = unquoted.match(/(true|false|[-]?\d+[\.]?\d*)/);
  if (parameters && parameters[0].length == unquoted.length) // matches, return exact
  {
    retVal = expression;
  }
  else
  {
    if (dwscripts.isQuoted(expression))
    {
      retVal = dwscripts.unescQuotes(unquoted);
    }
    else
    {
      retVal = "<%# " + expression + " %>";
    }
  }

  return retVal;
}


//--------------------------------------------------------------------
// FUNCTION:
//   hasServerMarkup
//
// DESCRIPTION:
//   This function returns true if the given expression contains
//   server markup.
//
// ARGUMENTS:
//   expression - string - the expression to test for server markup
//
// RETURNS:
//   boolean
//--------------------------------------------------------------------

function hasServerMarkup(expression)
{
  expression = expression.toString();
  return (expression.indexOf("<%") != -1 && expression.indexOf("%>") != -1);
}


//--------------------------------------------------------------------
// FUNCTION:
//   trimServerMarkup
//
// DESCRIPTION:
//   This function returns the given expression with any server markup
//   removed.
//
// ARGUMENTS:
//   expression - string - the expression to remove server markup from
//
// RETURNS:
//   string
//--------------------------------------------------------------------

function trimServerMarkup(expression)
{
  var retVal = expression.toString();

  if (retVal.length)
  {
    var begininlineindex = retVal.indexOf("<%#");
    var endinlineindex  =  retVal.indexOf("%>");
    if ((begininlineindex != -1) && (endinlineindex!=-1))
    {
      retVal = retVal.substring(begininlineindex+3, endinlineindex);
    }
    else
    {
      var begininlineindex = retVal.indexOf("<%");
      var endinlineindex  =  retVal.indexOf("%>");
      if ((begininlineindex != -1) && (endinlineindex!=-1))
      {
        retVal = retVal.substring(begininlineindex+2, endinlineindex);
      }
    }

    retVal = dwscripts.trim(retVal);
  }

  return retVal;
}

//--------------------------------------------------------------------
// FUNCTION:
//   findAllColumnNames
//
// DESCRIPTION:
//   Returns the column names for the given recordset.
//   If rs is not specified the column names for the first recordset
//   are returned.
//
// ARGUMENTS:
//   rs - string - optional - the name of a data source
//
// RETURNS:
//   array of strings
//--------------------------------------------------------------------

function findAllColumnNames(rs)
{
  var nameList = new Array();

  // Is it a "normal" recordset or from a stored procedure?

  var sbObjs = dwscripts.getServerBehaviorsByFileName("Recordset.htm");
  var sbObj = null;

  for (var i = 0; (i < sbObjs.length) && (sbObj == null); i++)
  {
    if (sbObjs[i].getRecordsetName() == rs)
    {
      sbObj = sbObjs[i];
    }
  }

  if (!sbObj)
  {
    sbObjs = dwscripts.getServerBehaviorsByFileName("StoredProc.htm");

    for (var i = 0; (i < sbObjs.length) && (sbObj == null); i++)
    {
      if (sbObjs[i].getRecordsetName() == rs)
      {
        sbObj = sbObjs[i];
      }
    }
  }

  if (sbObj)
  {
    var rsDOM = dreamweaver.getDocumentDOM(dreamweaver.getConfigurationPath() + "/DataSources/ASP.NET_Csharp/Recordset.htm");

    if (rsDOM)
    {
      nameList = rsDOM.parentWindow.generateDynamicSourceBindings(sbObj.getTitle());
    }
  }

  return nameList;
}

function getNullToken()
{
  return "null";
}

function getEqualsStatement(left, right)
{
  return (left + " == " + right);
}

function getTernaryStatement(condition, trueClause, falseClause)
{
  return condition + " ? " + trueClause + " : " + falseClause;
}

function getParameterAsInteger(param)
{
  return "Int32.Parse(" + param + ")";
}

function getParameterExpressionFromType(varType, varNameOrValue)
{
  var expr = null;
  var snytax = getParameterSyntaxFromType(varType, varNameOrValue);

  switch (varType)
  {
    case MM.LABEL_ASPNET_Param_Types[0]:
    case MM.LABEL_ASPNET_Param_Types[1]:
	  expr = "((" + snytax + " != null) && (" +  snytax + ".Length > 0))";
      break;

    case MM.LABEL_ASPNET_Param_Types[2]:
    case MM.LABEL_ASPNET_Param_Types[3]:
    case MM.LABEL_ASPNET_Param_Types[4]:
	  expr = "(" + snytax + " != null)";
      break;
  }

  return expr;
}

function getParameterSyntaxFromType(varType, varNameOrValue)
{
  var runtimeVal;

  switch (varType)
  {
    case MM.LABEL_ASPNET_Param_Types[0]:
      runtimeVal = "Request.QueryString[\"" + varNameOrValue + "\"]";
      break;

    case MM.LABEL_ASPNET_Param_Types[1]:
      runtimeVal = "Request.Form[\"" + varNameOrValue + "\"]";
      break;

    case MM.LABEL_ASPNET_Param_Types[2]:
      runtimeVal = "Request.Cookies[\"" + varNameOrValue + "\"]";
      break;

    case MM.LABEL_ASPNET_Param_Types[3]:
      runtimeVal = "Session[\"" + varNameOrValue + "\"]";
      break;

    case MM.LABEL_ASPNET_Param_Types[4]:
      runtimeVal = "Application[\"" + varNameOrValue + "\"]";
      break;

    case MM.LABEL_ASPNET_Param_Types[5]:
    {
      runtimeVal = varNameOrValue;

      if ((runtimeVal.charAt(0) != '"') && (runtimeVal.charAt(0) != '\''))
      {
        runtimeVal = '"' + runtimeVal + '"';
      }

      break;
    }

    default:
      runtimeVal = dwscripts.sprintf(MM.MSG_UnknownParamType, varType);
      break;
  }

  return runtimeVal;
}

function getParameterTypeFromCode(runtimeValue)
{
  var runtime = runtimeValue;
  var varType;
  var varNameOrValue = "";
  var varFilterType = "";
  var varFilterRuntime = "";
  var varDefault = "";
  var charSet = dw.getDocumentDOM().getCharSet();
  var doEncode = false;

  if (charSet == "iso-8859-1")
  {
    doEncode = true;
    runtime = dw.nativeToLatin1(runtime);
  }

  // If this is a "filter" parameter (used for a LIKE clause),
  // then it will contain a % before the runtime, after the runtime
  // or both. Deal with this before testing for the type of the
  // runtime.

  if (runtime.search(/['|"]%['|"]\s*\+\s*\(([\s\S]+)\)\s*\+\s*['|"]%['|"]/i) != -1)
  {
    varFilterType = "contains";
    varFilterRuntime = RegExp.$1;

    runtime = varFilterRuntime;
  }
  else if (runtime.search(/['|"]%['|"]\s*\+\s*\(([\s\S]+)\)/i) != -1)
  {
    varFilterType = "ends with";
    varFilterRuntime = RegExp.$1;

    runtime = varFilterRuntime;
  }
  else if (runtime.search(/\(([\s\S]+)\)\s*\+\s*['|"]%['|"]/i) != -1)
  {
    varFilterType = "begins with";
    varFilterRuntime = RegExp.$1;

    runtime = varFilterRuntime;
  }

  if (runtime.search(/\(\(Request\.QueryString\["([^"]*)"\] != null\) && \(Request\.QueryString\["([^"]*)"\]\.Length > 0\)\) \? ([^\r\n]*) : ([^\r\n]*)\s*/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[0];
  }
  else if (runtime.search(/\(\(Request\.Form\["([^"]*)"\] != null\) && \(Request\.Form\["([^"]*)"\]\.Length > 0\)\) \? ([^\r\n]*) : ([^\r\n]*)\s*/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[1];
  }
  else if (runtime.search(/\(Request\.Cookies\["([^"]*)"\] != null\) \? ([^\r\n]*) : ([^\r\n]*)\s*/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[2];
  }
  else if (runtime.search(/\(Session\["([^"]*)"\] != null\) \? ([^\r\n]*) : ([^\r\n]*)\s*/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[3];
  }
  else if (runtime.search(/\(Application\["([^"]*)"\] != null\) \? ([^\r\n]*) : ([^\r\n]*)\s*/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[4];
  }
  else
  {
    varType = MM.LABEL_ASPNET_Param_Types[5];
  }

  if (varType == MM.LABEL_ASPNET_Param_Types[5])
  {
    varNameOrValue = runtime;
  }
  else
  {
    varNameOrValue = RegExp.$1;
    varDefault = RegExp.$4;

    if (doEncode)
    {
      varNameOrValue = dw.latin1ToNative(varNameOrValue);
      varDefault = dw.latin1ToNative(varDefault);
    }
  }

  var outObj = new Object();

  outObj.varType = varType;
  outObj.varNameOrValue = dwscripts.trim(varNameOrValue);
  outObj.varDefault = dwscripts.trim(varDefault);
  outObj.varFilterType = varFilterType;
  outObj.varFilterRuntime = varFilterRuntime;

  return outObj;
}

function getParameterTypeFromExpression(expression)
{
  var runtimeVal = expression;
  var varType;
  var varNameOrValue = "";
  var charSet = dw.getDocumentDOM().getCharSet();
  var doEncode = false;

  if (charSet == "iso-8859-1")
  {
    doEncode = true;
    runtimeVal = dw.nativeToLatin1(runtimeVal);
  }

  if (runtimeVal.search(/\(\(Request\.QueryString\["([^"]*)"\] != null\) && \(Request\.QueryString\["([^"]*)"\]\.Length > 0\)\)/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[0];
  }
  else if (runtimeVal.search(/\(\(Request\.Form\["([^"]*)"\] != null\) && \(Request\.Form\["([^"]*)"\]\.Length > 0\)\)/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[1];
  }
  else if (runtimeVal.search(/\(Request\.Cookies\["([^"]*)"\] != null\)/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[2];
  }
  else if (runtimeVal.search(/\(Session\["([^"]*)"\] != null\)/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[3];
  }
  else if (runtimeVal.search(/\(Application\["([^"]*)"\] != null\)/i) != (-1))
  {
    varType = MM.LABEL_ASPNET_Param_Types[4];
  }
  else
  {
    varType = MM.LABEL_ASPNET_Param_Types[5];
  }

  if (varType == MM.LABEL_ASPNET_Param_Types[5])
  {
    varNameOrValue = runtimeValue;
  }
  else
  {
    varNameOrValue = RegExp.$1;

    if (doEncode)
    {
      varNameOrValue = dw.latin1ToNative(varNameOrValue);
    }
  }

  var outObj = new Object();

  outObj.varType = varType;
  outObj.varNameOrValue = dwscripts.trim(varNameOrValue);

  return outObj;
}

//--------------------------------------------------------------------
// FUNCTION:
//   escSqlQuotes
//
// DESCRIPTION:
//   For C#, single quotes in the sql string must be
//   replace with \u0027
//
// ARGUMENTS:
//   SQL statement as a string
//
// RETURNS:
//   string
//--------------------------------------------------------------------

function escSQLQuotes(sql)
{
  var retVal = sql;

  retVal =  retVal.replace(/'/g, "\\u0027");
  retVal = retVal.replace(/"/g, '\\"');

  return retVal;
}

function unescSQLQuotes(sql)
{
  var retVal = sql;

  retVal = retVal.replace(/\\u0027/g, "'");
  retVal = retVal.replace(/\\"/g, '"');

  return retVal;
}

